@@ -14,6 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import warnings
 from typing import List, Optional, Tuple
 
 import numpy as np
@@ -21,30 +22,35 @@ import scipy.sparse as sp
 
 import cvxpy.interface as intf
 import cvxpy.lin_ops.lin_utils as lu
-import cvxpy.utilities.eigvals as eig_util
+import cvxpy.settings as s
+import cvxpy.utilities.linalg as eig_util
 from cvxpy.expressions.leaf import Leaf
-from cvxpy.settings import EIGVAL_TOL
 from cvxpy.utilities import performance_utils as perf
 
+NESTED_LIST_WARNING = "Initializing a Constant with a nested list is " \
+                      "undefined behavior. Consider using a numpy array instead."
+
 
 class Constant(Leaf):
     """
     A constant value.
 
-    Raw numerical constants (Python primite types, NumPy ndarrays,
-    and NumPy matrices) are implicitly cast to constants via Expression
-    operator overloading. For example, if ``x`` is an expression and
-    ``c`` is a raw constant, then ``x + c`` creates an expression by
-    casting ``c`` to a Constant.
+    Raw numerical constants such as Python primitive types or NumPy ndarrays
+    are implicitly cast to constants via Expression operator overloading.
+    For example, if ``x`` is an expression and``c`` is a raw constant,
+    then ``x + c`` creates an expression by casting ``c`` to a Constant.
     """
 
-    def __init__(self, value) -> None:
+    def __init__(self, value, name: Optional[str] = None) -> None:
         # Keep sparse matrices sparse.
         if intf.is_sparse(value):
             self._value = intf.DEFAULT_SPARSE_INTF.const_to_matrix(
                 value, convert_scalars=True)
             self._sparse = True
         else:
+            if isinstance(value, list) and any(isinstance(i, list) for i in value):
+                warnings.warn(NESTED_LIST_WARNING)
+
             self._value = intf.DEFAULT_INTF.const_to_matrix(value)
             self._sparse = False
         self._imag: Optional[bool] = None
@@ -55,12 +61,23 @@ class Constant(Leaf):
         self._psd_test: Optional[bool] = None
         self._nsd_test: Optional[bool] = None
         self._cached_is_pos = None
+        self._skew_symm = None
+        self._name = name
         super(Constant, self).__init__(intf.shape(self.value))
 
     def name(self) -> str:
-        """The value as a string.
         """
-        return str(self.value)
+         The value of the constant as a string.
+        """
+        if self._name is None:
+            if len(self.shape) == 2 and "\n" in str(self.value):
+                return np.array2string(self.value,
+                                       edgeitems=s.PRINT_EDGEITEMS,
+                                       threshold=s.PRINT_THRESHOLD,
+                                       formatter={'float': lambda x: f'{x:.2f}'})
+            return str(self.value)
+        else:
+            return self._name
 
     def constants(self) -> List["Constant"]:
         """Returns self as a constant.
@@ -195,6 +212,11 @@ class Constant(Leaf):
         self._symm = is_symm
         self._herm = is_herm
 
+    def is_skew_symmetric(self) -> bool:
+        if self._skew_symm is None:
+            self._skew_symm = intf.is_skew_symmetric(self.value)
+        return self._skew_symm
+
     @perf.compute_once
     def is_psd(self) -> bool:
         """Is the expression a positive semidefinite matrix?
@@ -213,7 +235,7 @@ class Constant(Leaf):
 
         # Compute sign of bottom eigenvalue if absent.
         if self._psd_test is None:
-            self._psd_test = eig_util.is_psd_within_tol(self.value, EIGVAL_TOL)
+            self._psd_test = eig_util.is_psd_within_tol(self.value, s.EIGVAL_TOL)
 
         return self._psd_test
 
@@ -235,6 +257,6 @@ class Constant(Leaf):
 
         # Compute sign of top eigenvalue if absent.
         if self._nsd_test is None:
-            self._nsd_test = eig_util.is_psd_within_tol(-self.value, EIGVAL_TOL)
+            self._nsd_test = eig_util.is_psd_within_tol(-self.value, s.EIGVAL_TOL)
 
         return self._nsd_test
