@@ -498,10 +498,10 @@ class TestAtoms(BaseTest):
             cp.sum(Variable(2), axis=1).shape
         self.assertEqual(str(cm.exception), "axis 1 is out of bounds for array of dimension 1")
 
-        A = sp.eye(3)
+        A = sp.eye_array(3)
         self.assertEqual(cp.sum(A).value, 3)
 
-        A = sp.eye(3)
+        A = sp.eye_array(3)
         self.assertItemsAlmostEqual(cp.sum(A, axis=0).value, [1, 1, 1])
 
     def test_multiply(self) -> None:
@@ -736,7 +736,7 @@ class TestAtoms(BaseTest):
         with self.assertRaises(Exception) as cm:
             cp.diag(self.C)
         self.assertEqual(str(cm.exception),
-                         "Argument to diag must be a vector or square matrix.")
+                         "Argument to diag must be a 1-d array or 2-d square array.")
 
         # Test that diag is PSD
         w = np.array([1.0, 2.0])
@@ -775,7 +775,7 @@ class TestAtoms(BaseTest):
         with self.assertRaises(Exception) as cm:
             cp.trace(self.C)
         self.assertEqual(str(cm.exception),
-                         "Argument to trace must be a square matrix.")
+                         "Argument to trace must be a 2-d square array.")
 
     def test_trace_sign_psd(self) -> None:
         """Test sign of trace for psd/nsd inputs.
@@ -803,7 +803,7 @@ class TestAtoms(BaseTest):
         with self.assertRaises(Exception) as cm:
             cp.upper_tri(self.C)
         self.assertEqual(str(cm.exception),
-                         "Argument to upper_tri must be a square matrix.")
+                         "Argument to upper_tri must be a 2-d square array.")
 
     def test_vec_to_upper_tri(self) -> None:
         x = Variable(shape=(3,))
@@ -940,18 +940,13 @@ class TestAtoms(BaseTest):
         with self.assertRaises(Exception) as cm:
             cp.sum_largest(self.x, -1)
         self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
+                         "Second argument must be a positive number.")
 
         with self.assertRaises(Exception) as cm:
             cp.lambda_sum_largest(self.x, 2.4)
         self.assertEqual(str(cm.exception),
                          "First argument must be a square matrix.")
 
-        with self.assertRaises(Exception) as cm:
-            cp.lambda_sum_largest(Variable((2, 2)), 2.4)
-        self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
-
         with self.assertRaises(ValueError) as cm:
             cp.lambda_sum_largest([[1, 2], [3, 4]], 2).value
         self.assertEqual(str(cm.exception),
@@ -976,6 +971,17 @@ class TestAtoms(BaseTest):
         copy = atom.copy()
         self.assertTrue(type(copy) is type(atom))
 
+        # Test lambda_sum_largest with float k
+        A = np.array([[2.0, 0.0], [0.0, 1.0]])  # Eigenvalues: [2, 1]
+        expr = cp.lambda_sum_largest(A, 1.5)
+        expected = 2.0 + 0.5 * 1.0  # Largest + 0.5 * second largest
+        self.assertAlmostEqual(expr.value, expected)
+
+        # Test lambda_sum_smallest with float k
+        expr = cp.lambda_sum_smallest(A, 1.3)
+        expected = 1.0 + 0.3 * 2.0  # Smallest + 0.3 * second smallest
+        self.assertAlmostEqual(expr.value, expected)
+
         # Check that sum_largest is PWL so can be canonicalized as a QP.
         atom = cp.sum_largest(self.x, 2)
         assert atom.is_pwl()
@@ -988,23 +994,49 @@ class TestAtoms(BaseTest):
             prev_idx = np.argsort(-v)[:i]
             self.assertAlmostEqual(expr.value, v[prev_idx].sum())
 
+        # Test with float k values
+        v = np.array([5.0, 3.0, 8.0, 1.0, 6.0])
+        x = Constant(v)
+        # k=2.5 should give sum of 2 largest (8 + 6 = 14) + 0.5 * 3rd largest (0.5 * 5 = 2.5) = 16.5
+        expr = cp.sum_largest(x, 2.5)
+        expected = 8.0 + 6.0 + 0.5 * 5.0
+        self.assertAlmostEqual(expr.value, expected)
+
+        # k=1.7 should give largest (8) + 0.7 * 2nd largest (0.7 * 6 = 4.2) = 12.2
+        expr = cp.sum_largest(x, 1.7)
+        expected = 8.0 + 0.7 * 6.0
+        self.assertAlmostEqual(expr.value, expected)
+
+        # k=0.3 should give 0.3 * largest (0.3 * 8 = 2.4)
+        expr = cp.sum_largest(x, 0.3)
+        expected = 0.3 * 8.0
+        self.assertAlmostEqual(expr.value, expected)
+
     def test_sum_smallest(self) -> None:
         """Test the sum_smallest atom and related atoms.
         """
         with self.assertRaises(Exception) as cm:
             cp.sum_smallest(self.x, -1)
         self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
-
-        with self.assertRaises(Exception) as cm:
-            cp.lambda_sum_smallest(Variable((2, 2)), 2.4)
-        self.assertEqual(str(cm.exception),
-                         "Second argument must be a positive integer.")
+                         "Second argument must be a positive number.")
 
         # Check that sum_smallest is PWL so can be canonicalized as a QP.
         atom = cp.sum_smallest(self.x, 2)
         assert atom.is_pwl()
 
+        # Test with float k values
+        v = np.array([5.0, 3.0, 8.0, 1.0, 6.0])
+        x = Constant(v)
+        # k=2.5 should give sum of 2 smallest (1 + 3 = 4) + 0.5 * 3rd smallest (0.5 * 5 = 2.5) = 6.5
+        expr = cp.sum_smallest(x, 2.5)
+        expected = 1.0 + 3.0 + 0.5 * 5.0
+        self.assertAlmostEqual(expr.value, expected)
+
+        # k=1.7 should give smallest (1) + 0.7 * 2nd smallest (0.7 * 3 = 2.1) = 3.1
+        expr = cp.sum_smallest(x, 1.7)
+        expected = 1.0 + 0.7 * 3.0
+        self.assertAlmostEqual(expr.value, expected)
+
     def test_cvar(self) -> None:
         """Test the cvar atom and its use in a linear program."""
         # Check that CVaR is correctly computed
@@ -1065,6 +1097,7 @@ class TestAtoms(BaseTest):
         # Check that the CVaR constraint is satisfied
         cvar_value = cp.cvar(A @ x.value, beta).value
         self.assertTrue(cvar_value <= kappa)
+
     def test_index(self) -> None:
         """Test the copy function for index.
         """
@@ -1117,7 +1150,7 @@ class TestAtoms(BaseTest):
         with self.assertRaises(Exception) as cm:
             cp.conv([[0, 1], [0, 1]], self.x)
         self.assertEqual(str(cm.exception),
-                         "The arguments to conv must resolve to vectors.")
+                         "The arguments to conv must resolve to a 1-d array")
 
         # Test with parameter input.
         # https://github.com/cvxpy/cvxpy/issues/2218
@@ -1629,9 +1662,9 @@ class TestAtoms(BaseTest):
         # Test with matrices
         A = np.arange(4).reshape((2, 2))
 
-        with pytest.raises(ValueError, match="x must be a vector"):
+        with pytest.raises(ValueError, match="x must be a 1-d array"):
             cp.outer(A, d)
-        with pytest.raises(ValueError, match="y must be a vector"):
+        with pytest.raises(ValueError, match="y must be a 1-d array"):
             cp.outer(d, A)
 
         # allow 2D inputs once row-major flattening is the default
@@ -1711,7 +1744,7 @@ class TestAtoms(BaseTest):
         with self.assertRaises(ValueError) as cm:
             cp.partial_trace(X, dims=[2, 3], axis=0)
         self.assertEqual(str(cm.exception),
-                         "Only supports square matrices.")
+                         "partial_trace only supports 2-d square arrays.")
 
         X = cp.Variable((6, 6))
         with self.assertRaises(ValueError) as cm:
@@ -1770,7 +1803,7 @@ class TestAtoms(BaseTest):
         with self.assertRaises(ValueError) as cm:
             cp.partial_transpose(X, dims=[2, 3], axis=0)
         self.assertEqual(str(cm.exception),
-                         "Only supports square matrices.")
+                         "partial_transpose only supports 2-d square arrays.")
 
         X = cp.Variable((6, 6))
         with self.assertRaises(ValueError) as cm:
