@@ -7,7 +7,7 @@ import cvxpy as cp
 import cvxpy.error as error
 from cvxpy.tests.base_test import BaseTest
 
-SOLVER = cp.ECOS
+SOLVER = cp.CLARABEL
 
 
 class TestDcp(BaseTest):
@@ -300,6 +300,58 @@ class TestDcp(BaseTest):
         with pytest.raises(error.DPPError):
             problem.solve(cp.SCS, enforce_dpp=True, ignore_dpp=True)
 
+    def test_quad_over_lin(self) -> None:
+        """Test case with parameter in quad_over_lin."""
+        # Bug where the second argument to quad_over_lin
+        # was a parameter and the problem was solved
+        # as a cone program with a quadratic objective:
+        # https://github.com/cvxpy/cvxpy/issues/2433
+        x = cp.Variable()
+        p = cp.Parameter()
+
+        loss = cp.quad_over_lin(x,p) + x
+        prob = cp.Problem(
+            cp.Minimize(loss),
+        )
+
+        with warnings.catch_warnings():
+            # TODO(akshayka): Try to emit DPP problems in Dqcp2Dcp
+            warnings.filterwarnings('ignore', message=r'.*DPP.*')
+            p.value = 1
+            prob.solve(solver=cp.CLARABEL)
+            sol1 = x.value.copy()
+            p.value = 1000
+            prob.solve(solver=cp.CLARABEL)
+            sol2 = x.value.copy()
+            p.value = 1
+            prob.solve(solver=cp.CLARABEL)
+            sol3 = x.value.copy()
+            assert not np.isclose(sol1, sol2)
+            assert np.isclose(sol1, sol3)
+
+        # Cannot solve as a QP with DPP.
+        with pytest.raises(error.DPPError):
+            prob.solve(cp.OSQP, enforce_dpp=True)
+
+        # works for DPP + DGP
+        x = cp.Variable(2, pos = True)
+        y = cp.Parameter(pos = True, value = 1)
+        constraints = [x >= 1]
+
+        objective = cp.quad_over_lin(x,y)
+
+        prob = cp.Problem(cp.Minimize(objective), constraints)
+        sol1 = prob.solve(gp=True)
+        y.value = 2
+        sol2 = prob.solve(gp=True)
+        y.value = 1
+        sol3 = prob.solve(gp=True)
+        assert np.isclose(sol1, 2)
+        assert np.isclose(sol2, 1)
+        assert np.isclose(sol3, 2)
+
+
+
 
 class TestDgp(BaseTest):
     def test_basic_equality_constraint(self) -> None:
@@ -747,8 +799,8 @@ class TestDgp(BaseTest):
         np.testing.assert_almost_equal(w.value, np.array([4, 4]), decimal=3)
 
         alpha.value = [4.0, 4.0]
-        problem.solve(SOLVER, gp=True, enforce_dpp=True)
-        self.assertAlmostEqual(problem.value, 40)
+        problem.solve(cp.CLARABEL, gp=True, enforce_dpp=True)
+        self.assertAlmostEqual(problem.value, 40, places=3)
         np.testing.assert_almost_equal(h.value, np.array([20, 20]), decimal=3)
         np.testing.assert_almost_equal(w.value, np.array([1, 1]), decimal=3)
 
@@ -836,3 +888,25 @@ class TestDgp(BaseTest):
         expr = cp.gmatmul(A, x_par)
         self.assertFalse(expr.is_dgp(dpp=True))
         self.assertTrue(expr.is_dgp(dpp=False))
+
+
+class TestCallbackParam(BaseTest):
+    x = cp.Variable()
+    p = cp.Parameter()
+    q = cp.Parameter()
+
+    def test_callback_param(self) -> None:
+        callback_param = cp.CallbackParam(callback=lambda: self.p.value * self.q.value)
+        problem = cp.Problem(cp.Minimize(self.x), [self.x >= callback_param])
+        assert problem.is_dpp()
+        self.p.value = 1.0
+        self.q.value = 4.0
+        problem.solve()
+        self.assertAlmostEqual(self.x.value, 4.0)
+
+        self.p.value = 2.0
+        problem.solve()
+        self.assertAlmostEqual(self.x.value, 8.0)
+
+        with pytest.raises(NotImplementedError, match="Cannot set the value of a CallbackParam"):
+            callback_param.value = 1.0
\ No newline at end of file
