@@ -14,7 +14,5 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+from . import grad, shape, sign
 from .canonical import Canonical
-from . import grad
-from . import shape
-from . import sign
